/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.os.Bundle;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.TextView;

import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Artist;
import com.thingspace.cloud.sdk.list.item.File;
import com.thingspace.cloud.sdk.list.item.Genre;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.media.R;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewGridAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewGridHolder;
import com.thingspace.media.cache.ImageCache;
import com.thingspace.media.cache.ImageFetcher;
import com.thingspace.media.cache.SortPreference;
import com.thingspace.media.listeners.RecyclerViewClickListener;
import com.thingspace.media.tasks.GetArtistFilesTask;
import com.thingspace.media.tasks.GetGenreFilesTask;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.views.SpaceItemDecorator;

/**
 * Displays files for Artist and Genre.
 */
public class ArtistGenreFilesActivity extends BaseActivity implements RecyclerViewClickListener, OnBindViewInterface {

    public static final String EXTRA_CONTENT_TYPE = "content_type";
    public static final int CONTENT_TYPE_ARTIST = 1;
    public static final int CONTENT_TYPE_GENRE = 2;
    private static final int COLUMN_COUNT = 3;
    protected static ImageFetcher staticImageFetcher;
    private static Artist artist;
    private static Genre genre;
    protected RecyclerView recyclerView;
    protected TextView emptyView;
    private ItemList<File> itemList;
    private int contentType = 1;

    public static void setArtist(Artist aArtist) {
        artist = aArtist;
    }

    public static void setGenre(Genre aGenre) {
        genre = aGenre;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.resource_recycler_view);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setVisibility(View.VISIBLE);
        setSupportActionBar(toolbar);

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        emptyView = ((TextView) findViewById(R.id.emptyView));
        emptyView.setText(R.string.search_result_empty);

        GridLayoutManager manager = new GridLayoutManager(this, COLUMN_COUNT);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                // for thumbs
                return 1;
            }
        });

        recyclerView.setLayoutManager(manager);
        recyclerView.setHasFixedSize(true);
        recyclerView.addItemDecoration(new SpaceItemDecorator(getResources().getDimensionPixelSize(R.dimen.thumb_spacing), 3));

        contentType = getIntent().getIntExtra(EXTRA_CONTENT_TYPE, CONTENT_TYPE_ARTIST);

        retrieveFiles();
    }

    protected void retrieveFiles() {
        switch (contentType) {
            case CONTENT_TYPE_ARTIST:
                getSupportActionBar().setTitle(artist.getName());
                GetArtistFilesTask getArtistFilesTask = new GetArtistFilesTask(this, this, artist, SortPreference.getSortType(this));
                getArtistFilesTask.execute();
                break;
            case CONTENT_TYPE_GENRE:
                getSupportActionBar().setTitle(genre.getName());
                GetGenreFilesTask getGenreFilesTask = new GetGenreFilesTask(this, this, genre, SortPreference.getSortType(this));
                getGenreFilesTask.execute();
        }
        showProgressDialog(getString(R.string.loading));
    }

    @Override
    public void onResponse(int taskCode, int responseCode, Object response) {
        if (response != null && (taskCode == HubbleMediaTask.TASK_ARTIST_FILES || taskCode == HubbleMediaTask.TASK_GENRE_FILES)) {
            dismissProgressDialog();
            if (response instanceof ItemList) {
                itemList = (ItemList<File>) response;
                if (itemList.getCount() > 0) {
                    emptyView.setVisibility(View.GONE);
                    recyclerView.setVisibility(View.VISIBLE);
                    recyclerView.setAdapter(new RecyclerViewGridAdapter(this, R.layout.grid_item, itemList, cloudAPI, this, this));
                } else {
                    recyclerView.setVisibility(View.GONE);
                    emptyView.setVisibility(View.VISIBLE);
                    emptyView.setText(R.string.no_items);
                }
            }
            return;
        }

        super.onResponse(taskCode, responseCode, response);
    }

    @Override
    public void onListItemClicked(View view, int position) {
    }

    @Override
    public void onListItemLongClicked(View view, int position) {
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {

        if (item == null) {
            return;
        }

        File file = (File) item;
        getImageFetcher().setLoadingImage(R.drawable.music_icon);
        getImageFetcher().loadImage(file.getContentToken(), ((RecyclerViewGridHolder) holder).imageView, cloudAPI);
    }

    public ImageFetcher getImageFetcher() {
        if (staticImageFetcher == null) {
            ImageCache.ImageCacheParams cacheParams = new ImageCache.ImageCacheParams(this, "thumbs");
            cacheParams.setMemCacheSizePercent(this, 0.5f);

            staticImageFetcher = new ImageFetcher(this, getResources().getDimensionPixelSize(R.dimen.photo_item_size));
            staticImageFetcher.addImageCache(cacheParams);
        }
        return staticImageFetcher;
    }
}
