/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.TextView;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.cloud.sdk.list.item.PlaylistItem;
import com.thingspace.media.R;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewGridAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewGridHolder;
import com.thingspace.media.cache.ImageCache;
import com.thingspace.media.cache.ImageFetcher;
import com.thingspace.media.listeners.RecyclerViewClickListener;
import com.thingspace.media.tasks.AddToPlaylistTask;
import com.thingspace.media.tasks.DeletePlaylistItemTask;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.RetrievePlaylistItemsTask;
import com.thingspace.media.views.SpaceItemDecorator;

import java.util.ArrayList;

/**
 * Playlist items activity.
 */
public class PlaylistItemsActivity extends BaseActivity implements RecyclerViewClickListener, OnBindViewInterface {

    public static final String EXTRA_UID = "extra_uid";
    public static final String EXTRA_NAME = "extra_name";
    public static final String EXTRA_CONTENT_TYPE = "content_type";
    public static final int CONTENT_TYPE_IMAGE_VIDEO = 1;
    public static final int CONTENT_TYPE_IMAGE = 2;
    public static final int CONTENT_TYPE_VIDEO = 3;
    public static final int CONTENT_TYPE_MUSIC = 4;
    protected static final int ACTION_DELETE = 0;
    private static final int COLUMN_COUNT = 3;
    private static final int SELECTION_RESULT = 111;
    protected static ImageFetcher staticImageFetcher;
    protected RecyclerView recyclerView;
    protected TextView emptyView;
    protected String playlistUid;
    protected int contentType = 1;
    protected FloatingActionButton floatingActionButton;
    private ItemList<PlaylistItem> itemList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.resource_recycler_view);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setVisibility(View.VISIBLE);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(getIntent().getStringExtra(EXTRA_NAME));

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        emptyView = ((TextView) findViewById(R.id.emptyView));
        emptyView.setText(R.string.search_result_empty);

        GridLayoutManager manager = new GridLayoutManager(this, COLUMN_COUNT);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                // for thumbs
                return 1;
            }
        });

        recyclerView.setLayoutManager(manager);
        recyclerView.setHasFixedSize(true);
        recyclerView.addItemDecoration(new SpaceItemDecorator(getResources().getDimensionPixelSize(R.dimen.thumb_spacing), 3));

        playlistUid = getIntent().getStringExtra(EXTRA_UID);
        contentType = getIntent().getIntExtra(EXTRA_CONTENT_TYPE, CONTENT_TYPE_IMAGE_VIDEO);

        floatingActionButton = (FloatingActionButton) findViewById(R.id.fab);
        floatingActionButton.setVisibility(View.VISIBLE);
        floatingActionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                String query = null;
                String title = null;
                switch (contentType) {
                    case CONTENT_TYPE_MUSIC:
                        query = CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'";
                        title = "Music";
                        break;
                    case CONTENT_TYPE_IMAGE:
                        query = CloudAPI.Search.MIME_TYPE + " LIKE '%image%'";
                        title = "Photos";
                        break;
                    case CONTENT_TYPE_VIDEO:
                        query = CloudAPI.Search.MIME_TYPE + " LIKE '%video%'";
                        title = "Videos";
                        break;
                    case CONTENT_TYPE_IMAGE_VIDEO:
                        query = CloudAPI.Search.MIME_TYPE + " LIKE '%image%' OR " + CloudAPI.Search.MIME_TYPE + " LIKE '%video%'";
                        title = "Photos/Videos";

                }

                Intent intent = new Intent(PlaylistItemsActivity.this, SearchResultActivity.class);
                intent.putExtra(SearchResultActivity.EXTRA_QUERY, query);
                intent.putExtra(SearchResultActivity.EXTRA_VIEW_MODE, SearchResultActivity.VIEW_MODE_SELECTION);
                intent.putExtra(SearchResultActivity.EXTRA_TITLE, title);
                startActivityForResult(intent, SELECTION_RESULT);
            }
        });
        retrieveFiles();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == SELECTION_RESULT) {
            if (data == null) {
                return;
            }

            ArrayList<CharSequence> paths = data.getCharSequenceArrayListExtra(SearchResultActivity.RESULT_SELECTED_PATHS);
            if (paths == null) {
                return;
            }

            ArrayList<String> stringPaths = new ArrayList<>();
            for (CharSequence path : paths) {
                stringPaths.add((String) path);
            }

            AddToPlaylistTask addToPlaylistTask = new AddToPlaylistTask(PlaylistItemsActivity.this, PlaylistItemsActivity.this, cloudAPI, getPlaylistUid(), stringPaths);
            addToPlaylistTask.execute();
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    protected void retrieveFiles() {
        if (!TextUtils.isEmpty(getPlaylistUid())) {
            RetrievePlaylistItemsTask task = new RetrievePlaylistItemsTask(this, this, cloudAPI, getPlaylistUid());
            task.execute();
            showProgressDialog(getString(R.string.loading));
        }
    }

    @Override
    public void onResponse(int taskCode, int responseCode, Object response) {
        if (response != null && taskCode == HubbleMediaTask.TASK_PLAYLIST_ITEMS) {
            dismissProgressDialog();
            if (response instanceof ItemList) {
                itemList = (ItemList<PlaylistItem>) response;
                if (itemList.getCount() > 0) {
                    emptyView.setVisibility(View.GONE);
                    recyclerView.setVisibility(View.VISIBLE);
                    recyclerView.setAdapter(new RecyclerViewGridAdapter(this, R.layout.grid_item, itemList, cloudAPI, this, this));
                } else {
                    recyclerView.setVisibility(View.GONE);
                    emptyView.setVisibility(View.VISIBLE);
                    emptyView.setText(R.string.no_items);
                }
            }
            return;
        } else if (taskCode == HubbleMediaTask.TASK_ADD_TO_PLAYLIST) {
            if (response instanceof ArrayList) {
                ArrayList<String> returnedPaths = (ArrayList<String>) response;
                for (String path : returnedPaths) {
                    Log.d("===============>", "Added Path: " + path);
                }
                retrieveFiles();
            }
            return;
        } else if (response != null && ((boolean) response && taskCode == HubbleMediaTask.TASK_DELETE_PLAYLIST_ITEM)) {
            retrieveFiles();
            return;
        }

        super.onResponse(taskCode, responseCode, response);
    }

    @Override
    public void onListItemClicked(View view, int position) {
        PlaylistItem playlistItem = itemList.getItem(position);
        String mimeType = playlistItem.getMimeType();
        if (mimeType != null && mimeType.indexOf("image") != -1) {
            Intent intent = new Intent(this, PhotoActivity.class);
            intent.putExtra(PhotoActivity.EXTRA_ITEM_NUMBER, position);
            PhotoActivity.setItemList(itemList);
            startActivity(intent);
        }
    }

    @Override
    public void onListItemLongClicked(View view, int position) {
        PlaylistItem playlistItem = itemList.getItem(position);
        final String itemUid = playlistItem.getItemuid();
        AlertDialog.Builder operationDialog = new AlertDialog.Builder(this);
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(this, R.layout.list_item);

        arrayAdapter.add(getString(R.string.action_delete));

        operationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case ACTION_DELETE:
                        AlertDialog.Builder deletePopup = new AlertDialog.Builder(PlaylistItemsActivity.this);
                        deletePopup.setMessage(R.string.delete_operation)
                                .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        DeletePlaylistItemTask task = new DeletePlaylistItemTask(PlaylistItemsActivity.this, PlaylistItemsActivity.this, cloudAPI, getPlaylistUid(), itemUid);
                                        task.execute();
                                    }
                                }).setNegativeButton(getString(R.string.action_no), null).create();
                        deletePopup.show();
                }
            }
        });

        operationDialog.setTitle(getString(R.string.file_actions));
        operationDialog.create();
        operationDialog.show();
    }

    protected String getPlaylistUid() {
        return playlistUid;
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {

        if (item == null) {
            return;
        }

        PlaylistItem playlistItem = (PlaylistItem) item;
        loadImage(playlistItem.getMimeType(), playlistItem.getContentToken(), holder);
    }

    protected void loadImage(String mimeType, String contentToken, RecyclerView.ViewHolder holder) {
        if (mimeType.indexOf("audio") != -1) {
            getImageFetcher().setLoadingImage(R.drawable.music_icon);
        } else if (mimeType.indexOf("video") != -1) {
            getImageFetcher().setLoadingImage(R.drawable.videos_icon);
        } else if (mimeType.indexOf("image") != -1) {
            getImageFetcher().setLoadingImage(R.drawable.thumb_placeholder);
        } else if (mimeType.indexOf("directory") != -1) {
            getImageFetcher().setLoadingImage(R.drawable.folder_icon);
        } else {
            getImageFetcher().setLoadingImage(R.drawable.file_icon);
        }

        getImageFetcher().loadImage(contentToken, ((RecyclerViewGridHolder) holder).imageView, cloudAPI);
    }

    public ImageFetcher getImageFetcher() {
        if (staticImageFetcher == null) {
            ImageCache.ImageCacheParams cacheParams = new ImageCache.ImageCacheParams(this, "thumbs");
            cacheParams.setMemCacheSizePercent(this, 0.5f);

            staticImageFetcher = new ImageFetcher(this, getResources().getDimensionPixelSize(R.dimen.photo_item_size));
            staticImageFetcher.addImageCache(cacheParams);
        }
        return staticImageFetcher;
    }
}
