/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.app.DatePickerDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.media.R;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * Activity to display search UI
 */
public class SearchActivity extends BaseActivity implements View.OnClickListener, View.OnFocusChangeListener, DatePickerDialog.OnDateSetListener {

    private static final String SEARCH_DATE = CloudAPI.Search.CAPTURED_DATE;
    private EditText searchEditText;
    private Toolbar toolbar;
    private DatePickerDialog dialogFromDate, dialogToDate;
    private EditText editTextFromDate, editTextToDate;
    private Calendar calendarFrom, calendarTo;
    private CheckBox checkBoxAll, checkBoxPhotos, checkBoxVideos, checkBoxMusics;
    private SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    private EditText customSearchQuery, customSearchLimit;
    private Spinner customSearchSortBy, customSearchQueryType;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_search);

        toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        searchEditText = (EditText) findViewById(R.id.editTextSearch);

        searchEditText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                    performSearch();
                    return true;
                }
                return false;
            }
        });

        editTextFromDate = (EditText) findViewById(R.id.editTextFromDate);
        editTextToDate = (EditText) findViewById(R.id.editTextToDate);

        // Need both onFocus listener to open date picker on first click and onClick listener to open date picker on second click if already focused.
        editTextFromDate.setOnClickListener(this);
        editTextFromDate.setOnFocusChangeListener(this);
        editTextToDate.setOnClickListener(this);
        editTextToDate.setOnFocusChangeListener(this);

        calendarFrom = Calendar.getInstance();
        calendarTo = Calendar.getInstance();

        Calendar calendar = Calendar.getInstance();
        dialogFromDate = new DatePickerDialog(this, this, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH));
        dialogToDate = new DatePickerDialog(this, this, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH));

        checkBoxAll = (CheckBox) findViewById(R.id.checkBoxAll);
        checkBoxPhotos = (CheckBox) findViewById(R.id.checkBoxPhotos);
        checkBoxVideos = (CheckBox) findViewById(R.id.checkBoxVideos);
        checkBoxMusics = (CheckBox) findViewById(R.id.checkBoxMusics);

        // Custon Search
        customSearchSortBy = (Spinner) findViewById(R.id.spinner_search_sort);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.search_sort_options, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        customSearchSortBy.setAdapter(adapter);

        customSearchQueryType = (Spinner) findViewById(R.id.spinner_search_query_type);
        adapter = ArrayAdapter.createFromResource(this, R.array.search_query_type, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        customSearchQueryType.setAdapter(adapter);

        customSearchQuery = (EditText) findViewById(R.id.editText_custom_searchQuery);
        customSearchLimit = (EditText) findViewById(R.id.editText_searchLimit);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
//        getMenuInflater().inflate(R.menu.menu_search, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    public void onClickButtonSearch(View view) {
        performSearch();
    }

    public void performSearch() {
        if (!validateSearchParameters()) {
            return;
        }

        Intent intent = new Intent(this, SearchResultActivity.class);
        intent.putExtra(SearchResultActivity.EXTRA_QUERY, prepareQuerySQL());
        startActivity(intent);
    }

    private boolean validateSearchParameters() {

        // Date filter is optional
        if (!TextUtils.isEmpty(editTextFromDate.getText().toString())) {
            if (TextUtils.isEmpty(editTextToDate.getText().toString())) {
                Toast.makeText(this, R.string.search_date_empty, Toast.LENGTH_SHORT).show();
                return false;
            }
        } else {
            if (!TextUtils.isEmpty(editTextToDate.getText().toString())) {
                Toast.makeText(this, R.string.search_date_empty, Toast.LENGTH_SHORT).show();
                return false;
            }
        }

        return true;
    }

    private String prepareQueryLucene() {
        StringBuilder queryBuilder = new StringBuilder();

        String searchText = searchEditText.getText().toString();

        queryBuilder.append(CloudAPI.Search.NAME + ":\"" + searchText + "\"");

        // add date filter
        if (!TextUtils.isEmpty(editTextFromDate.getText().toString())) {
            queryBuilder.append(" AND ");

            String toDate = editTextToDate.getText().toString();
            try {
                Calendar c = Calendar.getInstance();
                c.setTime(simpleDateFormat.parse(editTextToDate.getText().toString()));
                c.add(Calendar.DATE, 1);
                toDate = simpleDateFormat.format(c.getTime());
            } catch (ParseException e) {
                e.printStackTrace();
            }

            queryBuilder.append(SEARCH_DATE + ":[" + editTextFromDate.getText().toString() + " TO " + toDate + "]");
        }


        // prepare mime types
        if (!checkBoxAll.isChecked()) {
            int totalChecks = 0;
            if (checkBoxPhotos.isChecked()) {
                totalChecks++;
            }
            if (checkBoxVideos.isChecked()) {
                totalChecks++;
            }
            if (checkBoxMusics.isChecked()) {
                totalChecks++;
            }

            if (totalChecks > 1) {
                queryBuilder.append(" AND (");
            } else if (totalChecks == 1) {
                queryBuilder.append(" AND ");
            }
            if (checkBoxPhotos.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*image*\"");

                if (checkBoxVideos.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*video*\"");
                }

                if (checkBoxMusics.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*audio*\"");
                }
            } else if (checkBoxVideos.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*video*\"");
                if (checkBoxMusics.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*audio*\"");
                }
            } else if (checkBoxMusics.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*audio*\"");
            }

            if (totalChecks > 1) {
                queryBuilder.append(")");
            }
        } else {
            queryBuilder.append(" AND (");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*image*\"");
            queryBuilder.append(" OR ");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*video*\"");
            queryBuilder.append(" OR ");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + ":\"*audio*\"");
            queryBuilder.append(")");
        }
        return queryBuilder.toString();
    }

    private String prepareQuerySQL() {
        StringBuilder queryBuilder = new StringBuilder();

        String searchText = searchEditText.getText().toString();
        if (searchText.equalsIgnoreCase("winter")) {
            queryBuilder.append(" (");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-12-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-01-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-02-%'");
            queryBuilder.append(")");
        } else if (searchText.equalsIgnoreCase("spring")) {
            queryBuilder.append(" (");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-03-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-04-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-05-%'");
            queryBuilder.append(")");
        } else if (searchText.equalsIgnoreCase("summer")) {
            queryBuilder.append(" (");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-06-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-07-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-08-%'");
            queryBuilder.append(")");
        } else if (searchText.equalsIgnoreCase("fall")) {
            queryBuilder.append(" (");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-09-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-10-%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(SEARCH_DATE + " LIKE '%-11-%'");
            queryBuilder.append(")");
        } else {
            queryBuilder.append(CloudAPI.Search.NAME + " LIKE '%" + searchText + "%'");

            // add date filter
            if (!TextUtils.isEmpty(editTextFromDate.getText().toString())) {
                queryBuilder.append(" AND (");

                String toDate = editTextToDate.getText().toString();
                try {
                    Calendar c = Calendar.getInstance();
                    c.setTime(simpleDateFormat.parse(editTextToDate.getText().toString()));
                    c.add(Calendar.DATE, 1);
                    toDate = simpleDateFormat.format(c.getTime());
                } catch (ParseException e) {
                    e.printStackTrace();
                }

                queryBuilder.append(SEARCH_DATE + " >= '" + editTextFromDate.getText().toString() + "%' AND " + SEARCH_DATE + " <= '" + toDate + "%')");
            }
        }

        // prepare mime types
        if (!checkBoxAll.isChecked()) {
            int totalChecks = 0;
            if (checkBoxPhotos.isChecked()) {
                totalChecks++;
            }
            if (checkBoxVideos.isChecked()) {
                totalChecks++;
            }
            if (checkBoxMusics.isChecked()) {
                totalChecks++;
            }

            if (totalChecks > 1) {
                queryBuilder.append(" AND (");
            } else if (totalChecks == 1) {
                queryBuilder.append(" AND ");
            }
            if (checkBoxPhotos.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%image%'");

                if (checkBoxVideos.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%video%'");
                }

                if (checkBoxMusics.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'");
                }
            } else if (checkBoxVideos.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%video%'");
                if (checkBoxMusics.isChecked()) {
                    queryBuilder.append(" OR ");
                    queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'");
                }
            } else if (checkBoxMusics.isChecked()) {
                queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'");
            }

            if (totalChecks > 1) {
                queryBuilder.append(")");
            }
        } else {
            queryBuilder.append(" AND (");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%image%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%video%'");
            queryBuilder.append(" OR ");
            queryBuilder.append(CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'");
            queryBuilder.append(")");
        }
        return queryBuilder.toString();
    }

    @Override
    public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
        if (view == dialogFromDate.getDatePicker()) {
            calendarFrom.set(year, monthOfYear, dayOfMonth);
            editTextFromDate.setText(simpleDateFormat.format(calendarFrom.getTime()));
        } else if (view == dialogToDate.getDatePicker()) {
            calendarTo.set(year, monthOfYear, dayOfMonth);
            editTextToDate.setText(simpleDateFormat.format(calendarTo.getTime()));
        }
    }

    @Override
    public void onClick(View v) {
        if (v == editTextFromDate) {
            dialogFromDate.show();
        } else if (v == editTextToDate) {
            dialogToDate.show();
        }
    }

    @Override
    public void onFocusChange(View v, boolean hasFocus) {
        if (hasFocus) {
            onClick(v);
        }
    }

    public void onCheckboxClicked(View view) {
        boolean checked = ((CheckBox) view).isChecked();

        switch (view.getId()) {
            case R.id.checkBoxAll:
                if (checked) {
                    checkBoxPhotos.setChecked(false);
                    checkBoxVideos.setChecked(false);
                    checkBoxMusics.setChecked(false);
                }
                break;
            default:
                checkBoxAll.setChecked(false);
        }
    }

    public void onClickButtonSearchCustom(View view) {
        Intent intent = new Intent(this, SearchResultActivity.class);
        String query = customSearchQuery.getText().toString();
        intent.putExtra(SearchResultActivity.EXTRA_QUERY, query);
        intent.putExtra(SearchResultActivity.EXTRA_QUERY_TYPE, customSearchQueryType.getSelectedItem().toString());
        intent.putExtra(SearchResultActivity.EXTRA_SORT_BY, customSearchSortBy.getSelectedItem().toString());
        int limit = TextUtils.isEmpty(customSearchLimit.getText()) ? -1 : Integer.parseInt(customSearchLimit.getText().toString());
        intent.putExtra(SearchResultActivity.EXTRA_LIMIT, limit);
        startActivity(intent);
    }
}