/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.TextView;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.callback.SearchCallback;
import com.thingspace.cloud.sdk.exception.SearchException;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Contact;
import com.thingspace.media.R;
import com.thingspace.media.adapters.GridCursorAdapter;
import com.thingspace.media.adapters.GridCursorAdapterContacts;
import com.thingspace.media.tasks.HubbleMediaTask;

/**
 * Verizon cloud activity to retrieve and display photos from Verizon cloud.
 */
public class VerizonCloudActivity extends BaseActivity {

    public static final String EXTRA_EXIT = "EXIT";

    private Toolbar toolbar;
    private GridView gridViewPhotos, gridViewMusic, gridViewContacts;
    private GridCursorAdapter gridCursorAdapterPhotos, gridCursorAdapterMusic, gridCursorAdapterContacts;
    private TextView titlePhotos, titleMusic, titleContacts;
    private ItemList itemListPhotos, itemListMusic, itemListContacts;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getIntent().getBooleanExtra(EXTRA_EXIT, false)) {
            finish();
            return;
        }
        setContentView(R.layout.activity_home);

        toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(getString(R.string.title_verizon_cloud));
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        gridViewPhotos = (GridView) findViewById(R.id.gridView_photos);
        gridViewPhotos.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                onClickCardViewPhotos(null);
            }
        });
        titlePhotos = (TextView) findViewById(R.id.textview_title_photos);
        titlePhotos.setText(getString(R.string.title_photos, getString(R.string.loading)));

        gridViewMusic = (GridView) findViewById(R.id.gridView_music);
        gridViewMusic.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                onClickCardViewMusic(null);
            }
        });
        titleMusic = (TextView) findViewById(R.id.textview_title_music);
        titleMusic.setText(getString(R.string.title_music, getString(R.string.loading)));

        gridViewContacts = (GridView) findViewById(R.id.gridView_contacts);
        gridViewContacts.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                onClickCardViewContacts(null);
            }
        });
        titleContacts = (TextView) findViewById(R.id.textview_title_contacts);
        titleContacts.setText(getString(R.string.title_contacts, getString(R.string.loading)));

        if (cloudAPI.hasAuthorization(this)) {
            onAuthorizationCompleted();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (itemListPhotos != null) {
            itemListPhotos.close();
        }

        if (itemListMusic != null) {
            itemListMusic.close();
        }

        if (itemListContacts != null) {
            itemListContacts.close();
        }
    }

    protected void onAuthorizationCompleted() {
        populateFullView();
        retrieveContacts();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    public void onClickCardViewPhotos(View view) {
        if (itemListPhotos == null) {
            return;
        }

        Intent intent = new Intent(this, ResourceTabActivity.class);
        startActivity(intent);
    }

    public void onClickCardViewMusic(View view) {
        if (itemListMusic == null) {
            return;
        }

        Intent intent = new Intent(this, MusicResourceTabActivity.class);
        startActivity(intent);
    }

    public void onClickCardViewContacts(View view) {
        if (itemListContacts == null) {
            return;
        }

        ContactsListActivity.setItemList(itemListContacts);
        Intent intent = new Intent(this, ContactsListActivity.class);
        startActivity(intent);
    }

    @Override
    public void onResponse(int taskCode, int responseCode, Object response) {
        super.onResponse(taskCode, responseCode, response);

        switch (taskCode) {
            case HubbleMediaTask.TASK_FULLVIEW:
            case HubbleMediaTask.TASK_DELTA:
                populateFullView();
                break;
            case HubbleMediaTask.TASK_CONTACTS:
                populateContacts(response);
        }
    }

    protected void populateFullView() {
        try {
            cloudAPI.search(this, CloudAPI.QueryType.SQL, CloudAPI.Search.MIME_TYPE + " LIKE '%image%' OR " + CloudAPI.Search.MIME_TYPE + " LIKE '%video%'", null, CloudAPI.Search.CREATED_DATE + " DESC", -1, new SearchCallback() {

                @Override
                public void onError(SearchException exception) {

                }

                @Override
                public void onSuccess(final ItemList itemList, boolean isOffline) {
                    populateMusic();
                    runOnUiThread(new Runnable() {

                        @Override
                        public void run() {
                            Log.e("VerizonCloudActivity", "Photos/Videos count: " + itemList.getCount());
                            if (itemList.getCount() > 0) {
                                gridCursorAdapterPhotos = new GridCursorAdapter(VerizonCloudActivity.this, R.layout.grid_item, itemList, cloudAPI, 4);
                                gridViewPhotos.setAdapter(gridCursorAdapterPhotos);
                                titlePhotos.setText(getString(R.string.title_photos, String.valueOf(itemList.getCount())));
                                itemListPhotos = itemList;
                                return;
                            }
                            titlePhotos.setText(getString(R.string.title_photos, "0"));
                        }
                    });
                }
            });
        } catch (IllegalArgumentException e) {
            Log.e(getClass().getSimpleName(), "Exception searching: " + e);
        }
    }

    private void populateMusic() {
        try {
            cloudAPI.search(this, CloudAPI.QueryType.SQL, CloudAPI.Search.MIME_TYPE + " LIKE '%audio%'", null, CloudAPI.Search.CREATED_DATE + " DESC", -1, new SearchCallback() {
                @Override
                public void onError(SearchException exception) {

                }

                @Override
                public void onSuccess(final ItemList itemList, boolean isOffline) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Log.e("VerizonCloudActivity", "Music count: " + itemList.getCount());
                            if (itemList.getCount() > 0) {
                                gridCursorAdapterMusic = new GridCursorAdapter(VerizonCloudActivity.this, R.layout.grid_item, itemList, cloudAPI, 4);
                                gridViewMusic.setAdapter(gridCursorAdapterMusic);
                                titleMusic.setText(getString(R.string.title_music, String.valueOf(itemList.getCount())));
                                itemListMusic = itemList;
                                return;
                            }
                            titleMusic.setText(getString(R.string.title_music, "0"));
                        }
                    });
                }
            });
        } catch (IllegalArgumentException e) {
            Log.e(getClass().getSimpleName(), "Exception searching: " + e);
        }
    }

    private void populateContacts(Object response) {
        if (response == null) {
            titleContacts.setText(getString(R.string.title_contacts, "0"));
            return;
        }

        itemListContacts = (ItemList<Contact>) response;
        gridCursorAdapterContacts = new GridCursorAdapterContacts(VerizonCloudActivity.this, R.layout.grid_item_contact, itemListContacts, cloudAPI, 4);
        gridViewContacts.setAdapter(gridCursorAdapterContacts);
        titleContacts.setText(getString(R.string.title_contacts, String.valueOf(itemListContacts.getCount())));
    }
}
