/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.dialog;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;

import com.thingspace.media.R;

import java.text.DecimalFormat;

/**
 * Dialog to display file information
 */
public class FileInfoDialog extends DialogFragment {

    public static final String TAG = "FileInfoDialog";
    public static final String FILE_NAME = "name";
    public static final String FILE_MIME = "mime";
    public static final String FILE_SIZE = "size";
    public static final String FILE_DATE = "date";
    public static final String FILE_WIDTH = "width";
    public static final String FILE_HEIGHT = "height";

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        // Use the Builder class for convenient dialog construction
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.fileinfo, null);
        builder.setView(dialogView);

        ((TextView) dialogView.findViewById(R.id.fileinfo_name)).setText(getArguments().getString(FILE_NAME));
        ((TextView) dialogView.findViewById(R.id.fileinfo_mime)).setText(getArguments().getString(FILE_MIME));
        if (getArguments().getString(FILE_SIZE) != null) {
            ((TextView) dialogView.findViewById(R.id.fileinfo_size)).setText(getFileSize(Long.parseLong(getArguments().getString(FILE_SIZE))));
        }
        ((TextView) dialogView.findViewById(R.id.fileinfo_date)).setText(retrieveDate(getArguments().getString(FILE_DATE)));
        ((TextView) dialogView.findViewById(R.id.fileinfo_resolution)).setText(getArguments().getString(FILE_WIDTH) + " X " + getArguments().getString(FILE_HEIGHT));

        builder.setPositiveButton("OK",
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        FileInfoDialog.this.dismiss();
                    }
                });

        // Create the AlertDialog object and return it
        return builder.create();
    }

    private String retrieveDate(String date) {
        if (date != null && date.indexOf("T") != -1) {
            return date.substring(0, date.indexOf("T"));
        }
        return null;
    }

    private String getFileSize(long size) {
        if (size <= 0)
            return "0";
        final String[] units = new String[]{"B", "KB", "MB", "GB", "TB"};
        int digitGroups = (int) (Math.log10(size) / Math.log10(1024));
        return new DecimalFormat("#.##").format(size / Math.pow(1024, digitGroups)) + " " + units[digitGroups];
    }
}
