/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.dialog;

import android.app.Activity;
import android.app.Dialog;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.thingspace.media.R;
import com.thingspace.media.cache.SortPreference;
import com.thingspace.media.listeners.SortOptionChangeListener;

/**
 * Sort options dialog
 */
public class SortDialog extends DialogFragment implements AdapterView.OnItemClickListener {

    /**
     * The Constant TAG.
     */
    public static final String TAG = "SortDialog";
    private String[] sortItemList;
    private SortOptionChangeListener sortOptionChangeListener;

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        sortOptionChangeListener = (SortOptionChangeListener) activity;
    }

    public Dialog onCreateDialog(Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        LayoutInflater inflater = getActivity().getLayoutInflater();

        View dialogView = inflater.inflate(R.layout.dialog_sort, null);
        builder.setView(dialogView);

        ListView listView = (ListView) dialogView.findViewById(R.id.listView);
        listView.setAdapter(new SortArrayAdapter());
        listView.setOnItemClickListener(this);

        AlertDialog dialog = builder.create();

        return dialog;
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        SortPreference.setSortPreference(getActivity(), sortItemList[position]);

        sortOptionChangeListener.onSortOptionChange();

        DialogFragment dialogFragment = (DialogFragment) getFragmentManager().findFragmentByTag(TAG);
        if (dialogFragment != null) {
            dialogFragment.dismiss();
        }
    }

    private class SortArrayAdapter extends ArrayAdapter<String> {

        public SortArrayAdapter() {
            super(getActivity(), R.layout.list_item_sort, getResources().getStringArray(R.array.sort_options));
            sortItemList = getResources().getStringArray(R.array.sort_options);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            View itemView = LayoutInflater.from(getActivity()).inflate(R.layout.list_item_sort, parent, false);
            TextView textView = (TextView) itemView.findViewById(R.id.sortTextView);
            textView.setText(sortItemList[position]);
            ImageView checkImage = (ImageView) itemView.findViewById(R.id.sortImageView);

            if (sortItemList[position].equals(SortPreference.getSortPreference(getActivity()))
                    || (SortPreference.getSortPreference(getActivity()).isEmpty() && position == 0)) {
                textView.setTypeface(textView.getTypeface(), Typeface.BOLD);
                checkImage.setVisibility(View.VISIBLE);
            } else {
                checkImage.setVisibility(View.INVISIBLE);
            }
            return itemView;
        }
    }
}
