/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.fragments;

import android.content.Intent;
import android.support.v7.widget.RecyclerView;
import android.view.View;

import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Genre;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.media.R;
import com.thingspace.media.activities.ArtistGenreFilesActivity;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewListAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewListHolder;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.RetrieveMusicGenresTask;

/**
 * Music Genre Fragment
 */
public class MusicGenreListFragment extends BaseFragment implements OnBindViewInterface {

    ItemList<Genre> musicGenreList;

    public void executeQuery() {
        RetrieveMusicGenresTask retrieveMusicGenresTask = new RetrieveMusicGenresTask(getActivity(), this, cloudAPI);
        retrieveMusicGenresTask.execute();
    }

    protected int getLayoutType() {
        return LAYOUT_TYPE_LIST;
    }

    protected RecyclerView.Adapter getRecyclerViewAdapter(ItemList itemList) {
        return new RecyclerViewListAdapter(R.layout.list_item, itemList, this, this);
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {
        ((RecyclerViewListHolder) holder).textView.setText(((Genre) item).getName());
    }

    @Override
    public void onResponse(int taskCode, int errorCode, Object response) {
        if (taskCode == HubbleMediaTask.TASK_MUSIC_GENRE) {
            if (response == null || !(response instanceof ItemList)) {
                return;
            }

            musicGenreList = (ItemList<Genre>) response;
            recyclerView.setAdapter(getRecyclerViewAdapter(musicGenreList));
            return;
        }

        super.onResponse(taskCode, errorCode, response);
    }

    @Override
    public void onListItemClicked(View view, int position) {
        ArtistGenreFilesActivity.setGenre(musicGenreList.getItem(position));
        Intent intent = new Intent(getActivity(), ArtistGenreFilesActivity.class);
        intent.putExtra(ArtistGenreFilesActivity.EXTRA_CONTENT_TYPE, ArtistGenreFilesActivity.CONTENT_TYPE_GENRE);
        startActivity(intent);
    }
}
