/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.fragments;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;

import com.thingspace.cloud.sdk.enums.PlaylistType;
import com.thingspace.cloud.sdk.enums.SortType;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Album;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.media.R;
import com.thingspace.media.activities.AlbumsFilesActivity;
import com.thingspace.media.activities.SearchResultActivity;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewListAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewListHolder;
import com.thingspace.media.cache.SortPreference;
import com.thingspace.media.dialog.CreatePlaylistDialog;
import com.thingspace.media.tasks.DeletePlaylistTask;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.RetrievePhotoAlbumsTask;
import com.thingspace.media.tasks.UpdatePlaylistDefinitionTask;

import java.util.ArrayList;

/**
 * Photos/Videos Albums list fragment
 */
public class PhotosAlbumsListFragment extends BaseFragment implements OnBindViewInterface {

    private static final int ACTION_UPDATE = 0;
    private static final int ACTION_UPDATE_DEFINITION = 1;
    private static final int ACTION_DELETE = 2;
    ItemList<Album> albumItemList;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        FloatingActionButton fab = (FloatingActionButton) view.findViewById(R.id.fab);
        fab.setVisibility(View.VISIBLE);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                CreatePlaylistDialog createPlaylistDialog = new CreatePlaylistDialog();
                Bundle bundle = new Bundle();
                bundle.putInt(CreatePlaylistDialog.EXTRA_PLAYLIST_TYPE, CreatePlaylistDialog.TYPE_IMAGE_VIDEO);
                bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_ACTION, CreatePlaylistDialog.ACTION_PLAYLIST_CREATE);
                createPlaylistDialog.setArguments(bundle);
                createPlaylistDialog.setTargetFragment(PhotosAlbumsListFragment.this, 111);
                CreatePlaylistDialog.setCloudAPI(cloudAPI);
                createPlaylistDialog.show(getFragmentManager(), "Create_playlist");
            }
        });

        return view;
    }

    public void executeQuery() {
        SortType sortType = SortType.NAME.setOrder(SortType.SortOrder.ASC);
        if (SortPreference.getSortPreference(getActivity()).equalsIgnoreCase(getActivity().getString(R.string.sort_date))) {
            sortType = SortType.CREATION_DATE.setOrder(SortType.SortOrder.ASC);
        }

        RetrievePhotoAlbumsTask retrievePhotoAlbumsTask = new RetrievePhotoAlbumsTask(getActivity(), this, cloudAPI, sortType);
        retrievePhotoAlbumsTask.execute();
        showProgressDialog(getString(R.string.loading));
    }

    protected int getLayoutType() {
        return LAYOUT_TYPE_LIST;
    }

    protected RecyclerView.Adapter getRecyclerViewAdapter(ItemList itemList) {
        return new RecyclerViewListAdapter(R.layout.list_item, itemList, this, this);
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {
        ((RecyclerViewListHolder) holder).textView.setText(((Album) item).getName());
    }

    @Override
    public void onListItemClicked(View view, int position) {
        Album album = albumItemList.getItem(position);
        AlbumsFilesActivity.setAlbum(album);
        Intent intent = new Intent(getActivity(), AlbumsFilesActivity.class);
        intent.putExtra(AlbumsFilesActivity.EXTRA_NAME, album.getName());
//        intent.putExtra(AlbumsFilesActivity.EXTRA_UID, album.getUid());
        intent.putExtra(AlbumsFilesActivity.EXTRA_SOURCE_TYPE, AlbumsFilesActivity.TYPE_ALBUM);
        int contentType = AlbumsFilesActivity.CONTENT_TYPE_IMAGE_VIDEO;
        if ("image".equalsIgnoreCase(album.getType())) {
            contentType = AlbumsFilesActivity.CONTENT_TYPE_IMAGE;
        } else if ("video".equalsIgnoreCase(album.getType())) {
            contentType = AlbumsFilesActivity.CONTENT_TYPE_VIDEO;
        }
        intent.putExtra(AlbumsFilesActivity.EXTRA_CONTENT_TYPE, contentType);
        startActivity(intent);
    }

    @Override
    public void onListItemLongClicked(View view, final int position) {
        AlertDialog.Builder operationDialog = new AlertDialog.Builder(getActivity());
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(getActivity(), R.layout.list_item);

        arrayAdapter.add(getString(R.string.action_update));
        arrayAdapter.add(getString(R.string.update_definition));
        arrayAdapter.add(getString(R.string.action_delete));

        operationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                final Album album = albumItemList.getItem(position);
                switch (which) {
                    case ACTION_UPDATE:
                    case ACTION_UPDATE_DEFINITION:
                        CreatePlaylistDialog updatePlaylistDialog = new CreatePlaylistDialog();
                        Bundle bundle = new Bundle();
                        bundle.putInt(CreatePlaylistDialog.EXTRA_PLAYLIST_TYPE, CreatePlaylistDialog.TYPE_IMAGE_VIDEO);
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_ACTION, which == ACTION_UPDATE ?
                                CreatePlaylistDialog.ACTION_PLAYLIST_UDPATE : CreatePlaylistDialog.ACTION_PLAYLIST_UDPATE_DEFINITION);
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_NAME, album.getName());
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_SELECTED_TYPE, album.getType());
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_UID, album.getUid());
                        updatePlaylistDialog.setArguments(bundle);
                        updatePlaylistDialog.setTargetFragment(PhotosAlbumsListFragment.this, 111);
                        CreatePlaylistDialog.setCloudAPI(cloudAPI);
                        updatePlaylistDialog.show(getFragmentManager(), "Update_playlist");
                        break;
                    case ACTION_DELETE:
                        AlertDialog.Builder deletePopup = new AlertDialog.Builder(getActivity());
                        deletePopup.setMessage(R.string.delete_operation)
                                .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        showProgressDialog(getString(R.string.loading));
                                        DeletePlaylistTask task = new DeletePlaylistTask(getActivity(), PhotosAlbumsListFragment.this, cloudAPI, album.getUid());
                                        task.execute();
                                    }
                                }).setNegativeButton(getString(R.string.action_no), null).create();
                        deletePopup.show();
                        break;
                    default:
                        break;

                }
            }
        });

        operationDialog.setTitle(getString(R.string.file_actions));
        operationDialog.create();
        operationDialog.show();
    }

    @Override
    public void onResponse(int taskCode, int errorCode, Object response) {
        dismissProgressDialog();
        if (taskCode == HubbleMediaTask.TASK_GET_PHOTO_ALBUMS) {
            if (response == null || !(response instanceof ItemList)) {
                return;
            }

            emptyView.setVisibility(View.GONE);
            albumItemList = (ItemList<Album>) response;
            recyclerView.setAdapter(getRecyclerViewAdapter(albumItemList));

            return;
        } else if ((taskCode == HubbleMediaTask.TASK_DELETE_PLAYLIST && (boolean) response) || taskCode == HubbleMediaTask.TASK_UPDATE_PLAYLIST_DEFINITION) {
            executeQuery();
        }
        super.onResponse(taskCode, errorCode, response);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (data != null && data.hasExtra(SearchResultActivity.RESULT_SELECTED_PATHS)
                && data.hasExtra(SearchResultActivity.EXTRA_PLAYLIST_TYPE)) {
            ArrayList<String> paths = data.getStringArrayListExtra(SearchResultActivity.RESULT_SELECTED_PATHS);
            int contentType = data.getIntExtra(SearchResultActivity.EXTRA_PLAYLIST_TYPE, 1);
            String name = data.getStringExtra(SearchResultActivity.EXTRA_NAME);
            String uid = data.getStringExtra(SearchResultActivity.EXTRA_UID);

            UpdatePlaylistDefinitionTask updatePlaylistDefinitionTask = new UpdatePlaylistDefinitionTask(getActivity(), PhotosAlbumsListFragment.this,
                    cloudAPI, uid, name, paths, getPlaylistType(contentType));
            updatePlaylistDefinitionTask.execute();
        }
    }

    public PlaylistType getPlaylistType(int contentType) {
        switch (contentType) {
            case AlbumsFilesActivity.CONTENT_TYPE_IMAGE:
                return PlaylistType.IMAGE;
            case AlbumsFilesActivity.CONTENT_TYPE_IMAGE_VIDEO:
                return PlaylistType.IMAGE_VIDEO;
            case AlbumsFilesActivity.CONTENT_TYPE_VIDEO:
                return PlaylistType.VIDEO;
            case AlbumsFilesActivity.CONTENT_TYPE_MUSIC:
                return PlaylistType.MUSIC;
            default:
                return PlaylistType.IMAGE;
        }
    }
}
