/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.content.Context;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.cloud.sdk.list.item.Playlist;
import com.thingspace.cloud.sdk.list.item.PlaylistItem;
import com.thingspace.media.listeners.ThingspaceResponseListener;

/**
 * Class to demonstrate an api call to delete a file from an existing playlist.
 */
public class DeletePlaylistItemTask extends HubbleMediaTask {

    private CloudAPI cloudAPI;
    private String uid, itemUid;

    /**
     * Delete a playlist file
     *
     * @param context                    context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param cloudAPI                   {@link CloudAPI} object to access Cloud api
     * @param uid                        Playlist unique identifier as obtained from {@link Playlist#getUid()}
     * @param itemUid                    Unique identifier of the file to be deleted from the playlist. Can be retrieved from {@link PlaylistItem#getItemuid()}.
     */
    public DeletePlaylistItemTask(Context context, ThingspaceResponseListener thingspaceResponseListener, CloudAPI cloudAPI, String uid, String itemUid) {
        super(context, thingspaceResponseListener);
        this.cloudAPI = cloudAPI;
        this.uid = uid;
        this.itemUid = itemUid;
    }

    protected Object executeAPIRequest(String... params) {
        /**
         * Cloud SDK method call to delete a file from an existing playlist
         */
        return cloudAPI.deletePlaylistItem(contextWeakReference.get(), uid, itemUid);
    }

    protected Object onCloudAPIException(CloudAPIException ex) {
        return false;
    }

    @Override
    protected int getTaskCode() {
        return TASK_DELETE_PLAYLIST_ITEM;
    }
}
