/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.content.Context;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.cloud.sdk.list.item.Playlist;
import com.thingspace.media.listeners.ThingspaceResponseListener;

/**
 * Class to demonstrate Cloud SDK api call to delete a playlist.
 */
public class DeletePlaylistTask extends HubbleMediaTask {

    private CloudAPI cloudAPI;
    private String uid;

    /**
     * Delete a playlist
     *
     * @param context                    context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param cloudAPI                   {@link CloudAPI} object to access Cloud api
     * @param uid                        Playlist unique identifier as obtained from {@link Playlist#getUid()}
     */
    public DeletePlaylistTask(Context context, ThingspaceResponseListener thingspaceResponseListener, CloudAPI cloudAPI, String uid) {
        super(context, thingspaceResponseListener);
        this.cloudAPI = cloudAPI;
        this.uid = uid;
    }

    protected Object executeAPIRequest(String... params) {
        /**
         * Cloud SDK method call to delete a playlist
         */
        return cloudAPI.deletePlaylist(contextWeakReference.get(), uid);
    }

    protected Object onCloudAPIException(CloudAPIException ex) {
        return false;
    }

    @Override
    protected int getTaskCode() {
        return TASK_DELETE_PLAYLIST;
    }
}
