/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.content.Context;

import com.thingspace.cloud.sdk.enums.SortType;
import com.thingspace.cloud.sdk.list.item.Artist;
import com.thingspace.media.listeners.ThingspaceResponseListener;

/**
 * Class to demonstrate Cloud SDK api to retrieve audio files for this artist from the Cloud.
 */
public class GetArtistFilesTask extends HubbleMediaTask {

    private Artist artist;
    private SortType sortType;

    /**
     * Retrieve all the audio files of this artist stored in the could.
     *
     * @param context                    context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param artist                     {@link Artist} object to access Artist files
     * @param sortType                   Sort order for the response list
     */
    public GetArtistFilesTask(Context context, ThingspaceResponseListener thingspaceResponseListener, Artist artist, SortType sortType) {
        super(context, thingspaceResponseListener);
        this.artist = artist;
        this.sortType = sortType;
    }

    protected Object executeAPIRequest(String... params) {
        /**
         * Cloud SDK api call to retrieve files for this artist.
         */
        return artist.getFiles(contextWeakReference.get(), sortType);
    }

    @Override
    protected int getTaskCode() {
        return TASK_ARTIST_FILES;
    }
}
