/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.content.Context;

import com.thingspace.cloud.sdk.enums.SortType;
import com.thingspace.cloud.sdk.list.item.Genre;
import com.thingspace.media.listeners.ThingspaceResponseListener;

/**
 * Class to demonstrate Cloud SDK api call to retrieve audio files for a genre from the Cloud.
 */
public class GetGenreFilesTask extends HubbleMediaTask {

    private Genre genre;
    private SortType sortType;

    /**
     * Retrieve all the audio files of this genre stored in the could.
     *
     * @param context                    context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param genre                      {@link Genre} object to access Genre files
     * @param sortType                   Sort order for the response list
     */
    public GetGenreFilesTask(Context context, ThingspaceResponseListener thingspaceResponseListener, Genre genre, SortType sortType) {
        super(context, thingspaceResponseListener);
        this.genre = genre;
        this.sortType = sortType;
    }

    protected Object executeAPIRequest(String... params) {
        /**
         * Cloud SDK api call to retrieve all files for this genre.
         */
        return genre.getFiles(contextWeakReference.get(), sortType);
    }

    @Override
    protected int getTaskCode() {
        return TASK_GENRE_FILES;
    }
}
