/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.content.Context;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.item.Playlist;
import com.thingspace.media.listeners.ThingspaceResponseListener;

/**
 * Class to demonstrate Cloud SDK api call to retrieve files from an existing playlist.
 */
public class RetrievePlaylistItemsTask extends HubbleMediaTask {

    private CloudAPI cloudAPI;
    private String playlistUid;

    /**
     *
     * @param context
     * @param cloudAPI
     */
    /**
     * Retrieve all the files from an existing playlist
     *
     * @param context                    context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param cloudAPI                   {@link CloudAPI} object to access ThingSpace api
     * @param playlistUid                Playlist uid as obtained from {@link Playlist#getUid()}
     */
    public RetrievePlaylistItemsTask(Context context, ThingspaceResponseListener thingspaceResponseListener, CloudAPI cloudAPI, String playlistUid) {
        super(context, thingspaceResponseListener);
        this.cloudAPI = cloudAPI;
        this.playlistUid = playlistUid;
    }

    protected Object executeAPIRequest(String... params) {
        /**
         * Cloud SDK api call to retrieve all files from an existing playlist
         */
        return cloudAPI.getPlaylistItems(contextWeakReference.get(), playlistUid, null, -1, -1);
    }

    @Override
    protected int getTaskCode() {
        return TASK_PLAYLIST_ITEMS;
    }
}
