/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.tasks;

import android.app.Activity;
import android.util.Log;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.exception.CloudAPIException;
import com.thingspace.media.listeners.ThingspaceResponseListener;

import java.lang.ref.WeakReference;

/**
 * Class to demonstrate Cloud SDK api call to upload a file to user's cloud storage account.
 */
public class UploadTask extends HubbleMediaTask {

    private WeakReference<Activity> activityWeakReference;
    private String uploadPath, fileName, filePath;
    private CloudAPI cloudAPI;

    /**
     * Upload a file to the cloud
     *
     * @param activity                   Context object
     * @param thingspaceResponseListener Response listener to receive cloud api response
     * @param uploadPath                 Path of the directory where the file is intended to be stored in the cloud
     * @param fileName                   File name
     * @param filePath                   Path of the directory where the file is stored on the device.
     * @param cloudAPI                   CloudAPI object to access Cloud apis
     */
    public UploadTask(Activity activity, ThingspaceResponseListener thingspaceResponseListener, String uploadPath, String fileName, String filePath, CloudAPI cloudAPI) {
        super(activity, thingspaceResponseListener);
        activityWeakReference = new WeakReference<>(activity);
        this.uploadPath = uploadPath;
        this.fileName = fileName;
        this.cloudAPI = cloudAPI;
        this.filePath = filePath;
    }

    @Override
    protected Boolean executeAPIRequest(String... params) {
        if (activityWeakReference.get() == null) {
            Log.e(getClass().getSimpleName(), "activity reference null");
            return false;
        }

        /**
         * Cloud SDK api call to upload a file to the cloud.
         */
        return cloudAPI.uploadFile(activityWeakReference.get(), uploadPath, fileName, filePath);
    }

    protected Object onCloudAPIException(CloudAPIException ex) {
        return false;
    }

    @Override
    protected int getTaskCode() {
        return TASK_UPLOAD;
    }
}
